/*
 *            Copyright (c) 1998-2000 by NETsilicon Inc.
 *
 *  This software is copyrighted by and is the sole property of
 *  NETsilicon.  All rights, title, ownership, or other interests
 *  in the software remain the property of NETsilicon.  This
 *  software may only be used in accordance with the corresponding
 *  license agreement.  Any unauthorized use, duplication, transmission,
 *  distribution, or disclosure of this software is expressly forbidden.
 *
 *  This Copyright notice may not be removed or modified without prior
 *  written consent of NETsilicon.
 *
 *  NETsilicon, reserves the right to modify this software
 *  without notice.
 *
 *  NETsilicon
 *  411 Waverley Oaks Road                  USA 781.647.1234
 *  Suite 227                               http://www.netsilicon.com
 *  Waltham, MA 02452                       AmericaSales@netsilicon.com
 *
 *************************************************************************
 *
 *     Module Name: boardParams.c
 *         Version: 1.00
 *   Original Date: 09/30/99
 *          Author: Charles Gordon
 *        Language: Ansi C
 * Compile Options:
 * Compile defines:
 *       Libraries:
 *    Link Options:
 *
 *    Entry Points: 	customizeUseDefaultParams(),
 			customizeReadDevBoardParams(),
 			customizeWriteDevBoardParams(),
 			customizeGetMACAddress(), 
 			customizeGetIPParameters(),
 			customizeSaveIPParameters(), 
 			customizeGetSerialNumber(),
 			customizeSaveSerialNumber(), 
 *
 * Description.
 * =======================================================================
 *  This file contains the routines that read and write the development
 *  board configuration parameters in NVRAM.  THE CODE IN THIS FILE MUST 
 *  BE CUSTOMIZED TO SUPPORT YOUR APPLICATION HARDWARE.  The default 
 *  implementation supports the NET+OS development boards. This code should 
 *  NOT be used in customer end products since it generates NETsilicon 
 *  specific Ethernet MAC addresses.  At the very least, the routine 
 *  customizeGetMACAddress() must be rewritten to generate MAC addresses 
 *  in the range assigned to the customer by the IEEE.
 *
 *
 * Edit Date/Ver   Edit Description
 * ==============  ===================================================
 *
 * JZW  07/27/00   In function NAReadDevBoardParams, if NVRAM is unavailable
 *                 for any reason, we default to the structure defaultParams. At
 *                 this contains hard-coded values. Customers (refer to SCR 1006)
 *                 would prefer to have the defaults default to the contents of
 *                 file appconf.h. Therefore I have changed the two references to
 *                 defaultParams to, instead, call UseDefaultParams, which pulls
 *                 values from appconf.h
 *                 I have chosen not to change the definition of defaultParams. The
 *                 contents will be overwritten upon the first call to UseDefaultParams
 * PMJ  05/02/01   Inserted a NAWriteDevBoardParams() of the default parameters
 *                 in function NAReadDevBoardParams() on a read checksum failure.
 * PMJ  05/23/01   Changed computeChecksum() to remove the presently stored checksum 
 *                 value from the computation.
 * PMJ  03/23/02   Changed NAReadDevBoardParams() to allow expansion, plus included  
 *                 relevant security parameters.
 * PMJ  03/29/02   Included NVRAM capabilities for SNTP Servers. 
 * PMJ  04/09/02   Included Kerberos Realm name in defaultParams.  Removed trailing 
 *                 zeros in defaultParams since they are included in function 
 *                 UseDefaultParams().
 * PMJ  04/10/02   Prevent writes to NVRAM in NAWriteDevBoardParams() if 
 *                 checkAppUseNvram() does not return true.  Note checkAppUseNvram()
 *                 returns true only when APP_USE_NVRAM (in appconf.h) is true.
 * DXL	04/16/02   Cleaned up variable declared but never reference; warnings generated by new compiler
 * LZ   10/21/02   Changed checkAppUseNvram() to NAGetAppUseNvram().
 * CHG  11/14/02   Ran into a problem where invalid configuration records would sometimes be used.
 *                 Apparently the checksum test passes if all fields are zero.  Changed
 *                 NAReadDevBoardParams() to reset the config to default values if the baud rate
 *                 field is zero.  It should never be zero and hangs the board if it is.
 * HOANG(intern) 04/15/03  Added comments to the functions. This is the new API and new comments, need someone to look
 *			   over to see if anything thing need to be change in the comments.
 * MLC  08/08/03   Defined time zone in NVRAM and added default values for time zone.
 */



#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <ctype.h>
#include "bsp.h"
#include "boardParams.h"
#include "narmnvrm.h"
#include "appconf_api.h"
#include "Npttypes.h"
#include "bootldr.h"
#include "serParam.h"
#include "narmapi.h"
#include "NAsntp.h"

#define DEFAULT_STANDARD_TIMEZONE   NASNTP_GMT_MINUS_5  /* Eastern standard time USA */
#define DEFAULT_DAYLIGHT_TIMEZONE   NASNTP_GMT_MINUS_4  /* Eastern Daylight saving time */
#define DEFAULT_START_DAYLIGHT_RULE "4.1.0/02:00:00"    /* April,1st week,sunday/2am */
#define DEFAULT_END_DAYLIGHT_RULE   "10.5.0/02:00:00"   /* October,last week,sunday/2am */
#define DEFAULT_SNTP_SYNC_TIME      18000 /* in seconds = 5 hours */

#define DEFAULT_SERIAL_NUMBER       "N99999999"
#define OLD_SERIAL_NUMBER_LENGTH    6
#define OLD_SERIAL_NUMBER_START     (BP_SERIAL_NUMBER_LENGTH - OLD_SERIAL_NUMBER_LENGTH)

#define ETHERNET_ADDRESS_LENGTH     6
#define SHORT_FORMAT_LENGTH         6
#define LONG_FORMAT_LENGTH          17

#define BAD_HEX_DIGIT               0xff

static BYTE const digiMacPrefix[] = {0x00, 0x40, 0x9d};

static unsigned long computeChecksum (devBoardParamsType *params);

/*ad*
 * The board parameters API is used to read and write configuration
 * settings for the development board that are stored in NVRAM.  
 * 
 * These settings are stored in a devBoardParamsType structure in 
 * persistent storage on the development board.  Generally, you 
 * will need to make significant changes to this API and to the
 * format of NVRAM storage in order to support your specific
 * application.  This API is only meant to support NET+OS development
 * boards and to serve as an example.
 *
 *
 * @category Board_Params
 * @since 6.0
 * @overview Development Board Parameters
 * @library bsp
 * @include "boardParams.h"
 * @platform all
 * @external
*ad*/
 


/*
 * The structure below contains hardcoded default settings for a Netsilicon's 
 * development board's startup parameters.   
 * These constant values will be overwritten with the ones which are defined 
 * in appconf.h when customizeUseDefaultParams() is called. 
 * 
 *  Notes:
 *
 *      THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD.  The implementation
 *      below is for NET+OS development boards only.
 *
 */


devBoardParamsType defaultParams;

#if NOT_USE_NOW   /* The following structure has to be removed because GNU compiler warning. */
{
    DEFAULT_SERIAL_NUMBER,                                              /* invalid serial number*/
    {
      0
    },
    10,                                                                 /* 5 second delay */
    9600,                                                               /* default baud rate for serial port*/
    0/*0x05291989*/,                                                    /* telnet magic number, see menu.c in \example\natelmenu application*/
    sizeof(devBoardParamsType),                                         /* used to sense expanded configuration */
    0,                                                                  /* need to compute checksum*/
    0,                                                                  /* PPP COM1 IP Address  */
    0,                                                                  /* PPP COM2 IP Address  */
    0,                                                                  /* PPP COM1 Peer IP Address  */
    0,                                                                  /* PPP COM2 Peer IP Address  */
    "\0",                                                               /* root Password (can be NULL)  */
    "\0",                                                               /* gateway admin account Password   */
    "\0",                                                               /* gateway Fixed Key (can be NULL)  */
    "\0",                                                               /* reserved (can be NULL)  */
    "\0",                                                               /* reserved (can be NULL)  */
    0,                                                                  /* reserved */
    "\0",                                                               /* reserved (can be NULL) */
    0xc0052929,                                                         /* SNTP Primary IP Address -- 192.5.41.41 tock.usno.navy.mil (Washington, DC)   */
    0x121a0469,                                                         /* SNTP Secondary IP Address -- 18.26.4.105 bonehed.lcs.mit.edu (Cambridge, MA) */
    {0x00, 0x40, 0x9d, 0x00, 0x00, 0x00},                               /* Ethernet MAC address*/
    bpDefaultDuplex,
    {
        DEFAULT_SNTP_SYNC_TIME, 
        DEFAULT_STANDARD_TIMEZONE, DEFAULT_DAYLIGHT_TIMEZONE, "\0", "\0"    /* Default time zone */
    },
    1
};

#endif

static boardParamsFreeFnType freeFn = NULL;
static boardParamsMallocFnType mallocFn = NULL;

/*ad*
 * Sets the malloc routine used by the board parameters API
 *
 * Use this function to set the malloc() function that will be used
 * by the board Parameters API to allocate memory.  This should almost
 * always be set to the standard C malloc() function.  The bootloader
 * uses a different version of this function to reduce its memory
 * footprint.
 *
 * @category Board_Params  
 * @since 6.0
 * @external
 *
 * @param bpMallocFn    pointer to malloc() function
 * @param bpFreeFn      pointer to free() function
*ad*/
void NABoardParamsSetMalloc(boardParamsMallocFnType bpMallocFn, boardParamsFreeFnType bpFreeFn)
{
    mallocFn = bpMallocFn;
    freeFn = bpFreeFn;
}


/*ad*
 * Used internally to allocate memory buffers
 *
 * Use the malloc function registered by NABoardParamsSetMalloc()
 * to allocate a buffer.  Return NULL if NABoardParamsSetMalloc()
 * has not been called.
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @param size          number of bytes to allocate
 *
 * @return  NULL        out of memory or mallocFn not setup
 * @return pointer      pointer to buffer
*ad*/
static void * bpMalloc(size_t size)
{
    void *result = NULL;
    
    if (mallocFn != NULL)
    {
        result = mallocFn(size);
    }
    
    return result;
}


/*ad*
 * Used internally to free memory buffers
 *
 * Use the free function registered by NABoardParamsSetMalloc()
 * to allocate a buffer.  Return NULL if NABoardParamsSetMalloc()
 * has not been called.
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @param buffer        pointer to memory to free
*ad*/
static void bpFree(void * buffer)
{
    if (freeFn)
    {
        freeFn(buffer);
    }
} 
    

/*ad*
 * Calculate a checksum using a 32-bit CRC algorithm.  This algorithm
 * replaces the original checksum method that did a simple summation.
 * The checksum calculation is done in two stages because the checksum
 * result is stored in the middle of the record and we don't want to
 * include it in the calculation.
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @param params    pointer to buffer with NVRAM parameters
 *
 * @return 32-bit CRC checksum
*ad*/
static unsigned long computeChecksum(devBoardParamsType * params)
{
    WORD32 accumulator = blCrc32(0, params, offsetof(devBoardParamsType, checksum));
    
    return (blCrc32(accumulator, ((unsigned char *)&(params->checksum)) + sizeof(params->checksum),
        sizeof(devBoardParamsType) - offsetof(devBoardParamsType, checksum) - sizeof(params->checksum)));
}
    


/*
 * Deprecated version of computeChecksum.  This version performs
 * a mathematical sum of all the words in the structure.  It is
 * used when reading data stored in NET+OS 5.0 format.
 */
static unsigned long compute500Checksum(devBoardParams500Type * params)
{
    int idx, size;
    unsigned long sum = 0;
    unsigned long *longParams = (unsigned long *) params;

    size = sizeof (devBoardParams500Type) / sizeof (unsigned long);

    /*  subtract off the checksum -- so when we add it later it cancels */
    sum = -params->checksum;

    for (idx = 0; idx < size; idx++)
    {
        sum += *longParams;
        longParams++;
    }

    return (0xffffffff - sum);
}



/*ad*
 * Deprecated version of function that calculates the MAC address from the
 * serial number.
 *
 * @category Board_Params 
 * @since 6.0
 * @param mac_addrp	Pointer to destination of ethernet MAC address. This pointer must point to
 *			storage containing at least 6 bytes.
 * @param serialNumber  Ponter to buffer containing serial number.
 *
*ad*/

static void convertSerialNumberToMacAddress(BYTE * mac_addrp, char * serialNumber)
{
    unsigned long serial_num;

    /* The first half of the ethernet mac address contains the
       number assigned to Netsilicon by IEEE. The second half is computed
       from the serial number. */ 

    mac_addrp[0] = 0x00; /* IEEE supplied address for NetSilicon */
    mac_addrp[1] = 0x40;
    mac_addrp[2] = 0xAF;

/*
 * Note that we only use the last 6 digits of the serial number.
 */
    serial_num = (serialNumber[OLD_SERIAL_NUMBER_START]-'0') * 10 + (serialNumber[OLD_SERIAL_NUMBER_START+1]-'0');
    serial_num = serial_num * 0x2710; 
    serial_num = serial_num + (serialNumber[OLD_SERIAL_NUMBER_START+2]-'0')*1000 + (serialNumber[OLD_SERIAL_NUMBER_START+3]-'0')*100 
        + (serialNumber[OLD_SERIAL_NUMBER_START+4]-'0') * 10 + (serialNumber[OLD_SERIAL_NUMBER_START+5]-'0'); 

    mac_addrp[3] = serial_num >> 13;
    mac_addrp[4] = (serial_num >> 5) & 0x000000FF;
    mac_addrp[5] = (serial_num & 0x0000001F) << 3;
}




#if (BSP_SERIAL_NUMBER_FORMAT == BSP_SERIAL_NUMBER_FORMAT_600)



/*ad*
 * Returns the Ethernet MAC address stored in NVRAM.
 *
 * @external
 * @category Board_Params
 * @since 6.0
 *
 * @param mac_addrp	Pointer to destination of ethernet MAC address. This pointer must point to
 *			storage containing at least 6 bytes.
 *
 * @return 0		Success
 * @return -1       Unable to read NVRAM
*ad*/

int customizeGetMACAddress(BYTE * mac_addrp)
{
    devBoardParamsType *params;
    int result;

    NA_ASSERT(mac_addrp != NULL);

	params = (devBoardParamsType *)bpMalloc(sizeof(devBoardParamsType));
	if (params == NULL)
	{
		return -1;
	}
    result = customizeReadDevBoardParams (params);             /* read params in NVRAM*/

    memcpy (mac_addrp, params->ethernetMac, sizeof params->ethernetMac);

	bpFree(params);
    
    return result;
}

/*ad*
 * retrieves the serial number stored in a NET+ARM development board's NVRAM. 
 *
 * @external
 * @category Board_Params
 * @since 6.0
 *
 * @param srln		Destination for stored serial number. This array must be a minimum of 10 bytes.
*ad*/

void customizeGetSerialNumber(char * srln)
{
    devBoardParamsType *params;

    NA_ASSERT(srln != NULL);
    
	params = (devBoardParamsType *)bpMalloc(sizeof (devBoardParamsType));
    if (params == NULL)
		return;

    customizeReadDevBoardParams (params);             /* read params in NVRAM*/

/*
 * First two bytes of serial number don't count.  Yet...
 */
    memcpy (srln, params->serialNumber, sizeof params->serialNumber);
	bpFree (params);
}


/*ad*
 * re-writes the serial number of NET+ARM development board.
 *
 * @external
 * @category Board_Params
 * @since 6.0
 *
 * @param srln		Character pointer to the new serial number. The serial number must be composed
 *			of 9 alpha-numeric characters.
*ad*/
void customizeSaveSerialNumber(char * srln)
{
    devBoardParamsType *params;

    NA_ASSERT(srln != NULL);
    NA_ASSERT(strlen(srln) == BP_SERIAL_NUMBER_LENGTH);

	params = (devBoardParamsType *)bpMalloc(sizeof (devBoardParamsType));
	if (params == NULL)
		return;
    
    customizeReadDevBoardParams (params);             /* read params in NVRAM*/
    strcpy (params->serialNumber, srln);
    customizeWriteDevBoardParams (params);
	bpFree(params);
}


#elif (BSP_SERIAL_NUMBER_FORMAT == BSP_SERIAL_NUMBER_FORMAT_500)

/*
 * Deprecated version of function that calculates the MAC address from the
 * serial number.
 *
 * param mac_addrp	Pointer to destination of ethernet MAC address. This pointer must point to
 *			storage containing at least 6 bytes.
 *
 * return 0		Success
 */

int customizeGetMACAddress(BYTE * mac_addrp)
{
    devBoardParamsType *params;

    NA_ASSERT(mac_addrp != NULL);

	params = (devBoardParamsType *)bpMalloc(sizeof(devBoardParamsType));
	if (params == NULL)
		return -1;
    customizeReadDevBoardParams (params);             /* read params in NVRAM*/
    convertSerialNumberToMacAddress (mac_addrp, params->serialNumber);
	bpFree(params);
    
    return 0;
}



/*
 * Deprecated version of function that works with 6 digit serial numbers.
 *
 *
 * @param srln		Destination for stored serial number. This array must be a minimum of 8 bytes.
 */

void customizeGetSerialNumber(char * srln)
{
    devBoardParamsType *params;

    NA_ASSERT(srln != NULL);
    
	params = (devBoardParamsType *)bpMalloc(sizeof(devBoardParamsType));
	if (params == NULL)
		return;

    customizeReadDevBoardParams (params);             /* read params in NVRAM*/

/*
 * Ignore first 3 characters of serial number.
 */
    memcpy (srln, &params->serialNumber[OLD_SERIAL_NUMBER_START], 6);
    srln[6] = 0;

	bpFree(params);
}




/*
 * Deprecated version of function that works with 6 digit serial numbers.
 *
 * @param srln		Character pointer to the new serial number. The serial number must be composed
 *			of 6 alpha-numeric characters.
 */
void customizeSaveSerialNumber(char * srln)
{
    devBoardParamsType *params;

    NA_ASSERT(srln != NULL);
    NA_ASSERT(strlen(srln) == OLD_SERIAL_NUMBER_LENGTH);
    
	params = (devBoardParamsType *)bpMalloc(sizeof (devBoardParamsType));
	if (params == NULL)
		return;
    customizeReadDevBoardParams (params);             /* read params in NVRAM*/
    strcpy (&params->serialNumber[OLD_SERIAL_NUMBER_START], srln);
    customizeWriteDevBoardParams (params);
	bpFree(params);
}


#endif

/*ad*
 * Set the Ethernet MAC address in NVRAM.
 *
 * @external
 * @category Board_Params
 * @since 6.0
 *
 * @param mac_addrp	Pointer to destination of ethernet MAC address. This pointer must point to
 *			storage containing at least 6 bytes.
 *
 * @return 0		Success
 * @return -1       Unable to read NVRAM
*ad*/

int customizeSetMACAddress(BYTE * mac_addrp)
{
    devBoardParamsType *params;
    int result;

    NA_ASSERT(mac_addrp != NULL);

	params = (devBoardParamsType *)bpMalloc(sizeof(devBoardParamsType));
	if (params == NULL)
		return -1;

    result = customizeReadDevBoardParams (params);             /* read params in NVRAM*/

    if (result == BP_SUCCESS)
    {
        memcpy (params->ethernetMac, mac_addrp, sizeof params->ethernetMac);
        result = customizeWriteDevBoardParams (params);
    }
    
	bpFree(params);
    return result;
}



/*ad*
 * Convert hex digit into a byte.
 *
 * @category Board_Params 
 * @since 6.0
 *
 * @param hexDigit  pointer to 2 chars with hex value
 *
 * @return digit    binary value of hex string
*ad*/
static BYTE convertHexDigit(char hexDigit)
{
    BYTE returnValue = BAD_HEX_DIGIT;
    static char const table[]="0123456789ABCDEF";
    unsigned int idx;
    
    for (idx = 0; idx < strlen(table); idx++)
    {
        if (table[idx] == toupper(hexDigit))
        {
            returnValue = idx;
            break;
        }
    }
    
    return returnValue;
}


/*ad*
 * This routine converts an Ethernet MAC address string into
 * the equivalent binary value.  The address string can be
 * in one of two forms.  It can be the traditional set of
 * 6 2-hexdigit values seperated by colons, or
 * it can be a 6 hexdigit value which is the last 3 bytes of
 * the MAC address (the first 3 are the Digi MAC ID).
 *
 * @param dest      pointer to destination buffer to store 
 *                  MAC address to
 * @param source    pointer to source buffer with character
 *                  string MAC address.
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @return BP_SUCCESS       converted string to MAC address
 * @return BP_FAILURE       string is not a valid MAC address
*ad*/
int NAConvertMacAddressString(BYTE * dest, char * source)
{
    int idx;
    BYTE hi, low;
    int result = BP_FAILURE;
    
    if (strlen(source) == SHORT_FORMAT_LENGTH)
    {
        /*
         * If length is 6 chars, then decode as short format.
         */
        memcpy(dest, digiMacPrefix, sizeof digiMacPrefix);
        for (idx = sizeof digiMacPrefix; idx < ETHERNET_ADDRESS_LENGTH; idx++)
        {
            hi = convertHexDigit(*source++);
            low = convertHexDigit(*source++);
            if ((hi == BAD_HEX_DIGIT) || (low == BAD_HEX_DIGIT))
            {
                goto done;
            }
            dest[idx] = (hi << 4) | low;
        }
        result = BP_SUCCESS;
    }
    else if (strlen(source) == LONG_FORMAT_LENGTH)
    {
        /*
         * If length is 17 chars, then decode as long format.
         */
        for (idx = 0; idx < ETHERNET_ADDRESS_LENGTH; idx++)
        {
            hi = convertHexDigit(*source++);
            low = convertHexDigit(*source++);
            if ((hi == BAD_HEX_DIGIT) || (low == BAD_HEX_DIGIT))
            {
                goto done;
            }
            dest[idx] = (hi << 4) | low;
            if ((idx < (ETHERNET_ADDRESS_LENGTH - 1)) && (*source != ':'))
            {
                goto done;
            }
            source++;
         }
         result = BP_SUCCESS;
    }

    done:
    return result;
}


/*ad*
 * This routine returns a default MAC address.  If there is one defined in appconf.h,
 * then it is returned.  Otherwise it returns a hard coded one.
 *
 * @category Board_Params  
 * @since 6.0
 *
 * @return Ethernet MAC address
*ad*/
static BYTE * getDefaultMacAddress(void)
{
    char *value= NAGetAppBspMacAddress();
    static BYTE returnValue[ETHERNET_ADDRESS_LENGTH] = {0};

    memcpy (returnValue, digiMacPrefix, sizeof digiMacPrefix);
    if (value != NULL)
    {
        NAConvertMacAddressString(returnValue, value);
    }
    
    return returnValue;
}



/*ad*
 * Changes the params with the setting in the appconf.h
 *
 * @external
 * @category Board_Params
 * @since 6.0
 *
 * @param params 	Pointer to structure with development board parameters
 *
 * @return 0		Successfully changed to default setting
 * @return -1		Unable to read parameters
 *
 * @see @link devBoardParamsType
 *
 * @note	This code must be customized for your board should you decide not to use the
 *		setting in appconf.h. You may use the settings in structure defaultParams, or
 *		any other ways.
*ad*/

int customizeUseDefaultParams(devBoardParamsType * params)
{
    unsigned long app_ip_address_val=0;
    char	*valueStr;

    NA_ASSERT(params != NULL);
    if (params == NULL)
    {
        return -1;
    }
    
    /*  default the structure to zeros  */
    memset (params, 0, sizeof(devBoardParamsType));

    /* Get default network configuration*/
    customizeAceGetDefaultConfig(&params->aceConfig);

    params->waitTime = NAGetAppBspDelay ();
    params->baudrate = NAGetAppBspBaudRate();
    params->defaultDuplexLevel = NAGetDefaultEthernetDuplex();
    
    /*
     * Serial number in appconf.h may be wrong length, or nonexistent.  Write default
     * serial number, then copy in the serial number from appconf.h.  Since the length
     * may be wrong, right justify it.
     */
    strcpy(params->serialNumber, DEFAULT_SERIAL_NUMBER);
    valueStr = NAGetAppBspSerialNum();
    if (valueStr)
    {
        if (strlen(valueStr) < BP_SERIAL_NUMBER_LENGTH)
        {
            strcpy(&params->serialNumber[BP_SERIAL_NUMBER_LENGTH - strlen(valueStr)], valueStr);
        }
    }
    memcpy(params->ethernetMac, getDefaultMacAddress(), sizeof(params->ethernetMac));

    if (NAGetAppEnablePPP1 ())
    {
        /*  set the default ppp ip address  */
    	valueStr = NAGetAppPPP1Address ();
        app_ip_address_val      = NAInet_addr (valueStr);
    	params->ipAddrPpp1      = app_ip_address_val;

        /*  set the default ppp peer ip address  */
	    valueStr = NAGetAppPPP1PeerAddress ();
        app_ip_address_val      = NAInet_addr (valueStr);
    	params->ipAddrPpp1peer  = app_ip_address_val;
    }
    else
    {
    	params->ipAddrPpp1      = app_ip_address_val;
    	params->ipAddrPpp1peer  = 0;
    }

    if (NAGetAppEnablePPP2 ())
    {
        /*  set the default ppp ip address  */
	    valueStr = NAGetAppPPP2Address ();
        app_ip_address_val      = NAInet_addr(valueStr);
    	params->ipAddrPpp2      = app_ip_address_val;

        /*  set the default ppp peer ip address  */
	    valueStr = NAGetAppPPP2PeerAddress ();
        app_ip_address_val      = NAInet_addr(valueStr);
    	params->ipAddrPpp2peer  = app_ip_address_val;
    }
    else
    {
    	params->ipAddrPpp2      = 0;
    	params->ipAddrPpp2peer  = 0;
    }

	if (NAGetAppUseNetworkTimeProtocol ())
	{
		if ((valueStr = NAGetAppSntpPrimary ()) != NULL)
		{
		    params->ipAddrSntp1 = (unsigned long)NAInet_addr (valueStr);
		}

		if ((valueStr = NAGetAppSntpSecondary ()) != NULL)
		{
		    params->ipAddrSntp2 = (unsigned long)NAInet_addr (valueStr);
		}	
	}

	if ((valueStr = NAGetAppRootPassword ()) != NULL)
	{
	    memcpy(params->rootPassword, valueStr, NASYSACC_STRLEN_PASSWORD);
    	    params->rootPassword[NASYSACC_STRLEN_PASSWORD - 1] = 0;
	}

	if ((valueStr = NAGetAppAdministratorsAccount ()) != NULL)
	{
	    memcpy(params->adminPassword, valueStr, NASYSACC_STRLEN_PASSWORD);
    	    params->adminPassword[NASYSACC_STRLEN_PASSWORD - 1] = 0;
	}


    /* USA - Standard Eastern Time Zone */
    params->timezone.sync_time          = DEFAULT_SNTP_SYNC_TIME;
    params->timezone.std_zone_offset    = DEFAULT_STANDARD_TIMEZONE;
    params->timezone.dst_zone_offset    = DEFAULT_DAYLIGHT_TIMEZONE;
    strcpy(params->timezone.dst_start_rule, DEFAULT_START_DAYLIGHT_RULE);
    strcpy(params->timezone.dst_end_rule, DEFAULT_END_DAYLIGHT_RULE);
    return (0);
}


/*ad*
 * Determine if the parameter buffer passed to us looks like a valid set
 * of old style parameters.  Check the checksum, serial number, and
 * baud rate.
 *
 * @category Board_Params 
 * @since 6.0
 *
 * @param params    pointer to buffer with data that may be in 5.00 format.
 *
 * @return TRUE     record appears to be valid NET+OS 5.00 parameters
 * @return FALSE    record is invalid
*ad*/
static BOOLEAN isValidOldStyleParams(devBoardParams500Type * const params)
{
    BOOLEAN isValid = FALSE;
    unsigned int idx;
    
    if (compute500Checksum(params) == params->checksum) 
    {
        switch (params->baudrate)
        {
            case SIO_2400_BAUD:   
            case SIO_4800_BAUD:   
            case SIO_7200_BAUD:   
            case SIO_9600_BAUD:   
            case SIO_14400_BAUD:  
            case SIO_19200_BAUD:  
            case SIO_28800_BAUD:  
            case SIO_38400_BAUD:  
            case SIO_57600_BAUD:  
            case SIO_115200_BAUD: 
            case SIO_230400_BAUD: 
                isValid = TRUE;
                for (idx = 0; idx < sizeof(params->serialNumber); idx++)
                {
                    if (isdigit(params->serialNumber[idx]) == FALSE)
                    {
                        isValid = FALSE;
                        break;
                    }
                }
                break;
            default:
                break;
        }
        
    }
    
    return isValid;
}
    
/*ad*
 * Try to convert data stored in the old NET+OS 5.0 NVRAM format into 6.0 format.
 *
 * @category Board_Params 
 * @since 6.0
 *
 * @param params    pointer to buffer with data possible in old format that we
 *                  want to convert to the 6.00 format
 *
 * @return BP_FAILURE   Buffer pointed to by params is not in NET+OS 5.0/5.1 format.
 * @return BP_SUCCESS   Buffer has been converted.
*ad*/
static int convertFrom500Params(devBoardParamsType * params)
{
    devBoardParams500Type *oldStyle;
    int returnValue = BP_FAILURE;
    
	oldStyle = (devBoardParams500Type *)bpMalloc(sizeof(devBoardParams500Type));
	if (oldStyle == NULL)
		return BP_FAILURE;

    memcpy (oldStyle, params, sizeof(devBoardParams500Type));
    
    if (isValidOldStyleParams(oldStyle))
    {
        customizeUseDefaultParams(&defaultParams);

        params->serialNumber[0] = BP_OLDSTYLE_PREFIX;
        memcpy (&params->serialNumber[1], oldStyle->serialNumber, sizeof oldStyle->serialNumber);
        params->serialNumber[BP_SERIAL_NUMBER_LENGTH] = 0;
        customizeAceGetDefaultConfig(&params->aceConfig);
        params->aceConfig.ace_interface_info[0].static_config.isEnabled = !oldStyle->useDhcp;
        params->aceConfig.ace_interface_info[0].static_config.isConfigValid = TRUE;
        params->aceConfig.ace_interface_info[0].static_config.ip_address = oldStyle->ipAddress;
        params->aceConfig.ace_interface_info[0].static_config.subnet_mask = oldStyle->subnetMask;
        params->aceConfig.ace_interface_info[0].static_config.gateway = oldStyle->gateway;
        params->aceConfig.ace_interface_info[0].static_config.auto_assign = !oldStyle->useDhcp;
        params->aceConfig.ace_interface_info[0].dhcp_config.isEnabled = oldStyle->useDhcp;
        params->waitTime = oldStyle->waitTime;                            
        params->baudrate = oldStyle->baudrate;                            
        params->telnetMagic = oldStyle->telnetMagic;                        
        params->ipAddrPpp1 = oldStyle->ipAddrPpp1;                           
        params->ipAddrPpp2 = oldStyle->ipAddrPpp2;                           
        params->ipAddrPpp1peer = oldStyle->ipAddrPpp1peer;                       
        params->ipAddrPpp2peer = oldStyle->ipAddrPpp2peer;                       
        memcpy(params->rootPassword, oldStyle->rootPassword, sizeof params->rootPassword);        
        memcpy(params->adminPassword, oldStyle->adminPassword, sizeof params->adminPassword);       
        memcpy(params->FixedKey, oldStyle->FixedKey, sizeof params->FixedKey);                                   
        memcpy(params->resvfieldone, oldStyle->resvfieldone, sizeof params->resvfieldone);     
        memcpy(params->resvfieldtwo, oldStyle->resvfieldtwo, sizeof params->resvfieldtwo);    
        params->resvfieldthree = oldStyle->resvfieldthree;                      
        memcpy(params->resvfieldfour, oldStyle->resvfieldfour, sizeof params->resvfieldfour);
        params->ipAddrSntp1 = oldStyle->ipAddrSntp1;                         
        params->ipAddrSntp2 = oldStyle->ipAddrSntp2;
        convertSerialNumberToMacAddress(params->ethernetMac, params->serialNumber);
        params->defaultDuplexLevel = NAGetDefaultEthernetDuplex();
        
        memcpy(&params->wlnParams, &defaultParams.wlnParams, sizeof params->wlnParams);
        params->snmpEngineBoots = defaultParams.snmpEngineBoots;
        
        returnValue = BP_SUCCESS;
    }
    
	bpFree(oldStyle);
    return returnValue;
}

/* calculates the CRC for the specified size of board param data
   assuming initial fields upto ipAddrPpp1 are same as in new data */
static unsigned long isValidOlderVersionParams(devBoardParamsType * params, unsigned long size)
{
    WORD32 accumulator = blCrc32(0, params, offsetof(devBoardParamsType, checksum));
    
    return (blCrc32(accumulator, ((unsigned char *)&(params->checksum)) + sizeof(params->checksum),
        size - offsetof(devBoardParamsType, checksum) - sizeof(params->checksum)));
}

/* converting from 6.0? */
static int convertFrom60Params(devBoardParamsType * params)
{
    int ret = BP_FAILURE;
    devBoardParams60Type *params60 = (devBoardParams60Type *)params;
    WORD32 accumulator;
    unsigned int len;
    unsigned long chksum;
    
    /* checksum ok? */
    /* wln_params->band was not there in 6.0 */
    len = ((sizeof *params60)-sizeof (params->wlnParams.band)) - offsetof(devBoardParams60Type, checksum) - sizeof(params60->checksum);
    accumulator = blCrc32(0, params60, offsetof(devBoardParams60Type, checksum));
    chksum = blCrc32(accumulator, ((unsigned char *)&(params60->checksum)) + sizeof(params60->checksum), len);

    if (params60->checksum == chksum)
    {
        len = offsetof(devBoardParams60Type, wlnParams) - offsetof(devBoardParams60Type, waitTime);
        if (len < sizeof *params60) /* field position changed? */
        {
            memmove(&params->waitTime, &params60->waitTime, len);  
            memmove(&params->wlnParams, &params60->wlnParams, sizeof params->wlnParams);
            params->snmpEngineBoots = defaultParams.snmpEngineBoots;
            ret = BP_SUCCESS;
        }
    }

    return ret;    
}


/* converting from 6.1? */
static int convertFrom61Params(devBoardParamsType * params)
{
    int ret = BP_FAILURE;
    devBoardParams61Type *params61 = (devBoardParams61Type *)params;
    WORD32 accumulator;
    unsigned int len;
    unsigned long chksum;
    
    /* checksum ok? */
    len = (sizeof *params61) - offsetof(devBoardParams61Type, checksum) - sizeof(params61->checksum);
    accumulator = blCrc32(0, params61, offsetof(devBoardParams61Type, checksum));
    chksum = blCrc32(accumulator, ((unsigned char *)&(params61->checksum)) + sizeof(params61->checksum), len);

    if (params61->checksum == chksum)
    {
        len = (sizeof *params61) - offsetof(devBoardParams61Type, waitTime);
        if (len < sizeof *params61) /* field position changed? */
        {
            memmove(&params->waitTime, &params61->waitTime, len);  
            memcpy(&params->wlnParams, &defaultParams.wlnParams, sizeof params->wlnParams);
            params->snmpEngineBoots = defaultParams.snmpEngineBoots;
            ret = BP_SUCCESS;
        }
    }

    return ret;    
}

/* converting from 6.2? */
static int convertFrom62Params(devBoardParamsType * params)
{
    int ret = BP_FAILURE;
    devBoardParams62Type *params62 = (devBoardParams62Type *)params;
    unsigned long recSize = params62->sizeOfRecord;
    WORD32 accumulator;
    unsigned int len;
    unsigned long chksum;
    
    /* checksum ok? */
    if ((recSize <= sizeof *params62) && (recSize > offsetof(devBoardParams62Type, snmpEngineBoots)))
    {
        len = recSize - offsetof(devBoardParams62Type, checksum) - sizeof(params62->checksum);
        accumulator = blCrc32(0, params62, offsetof(devBoardParams62Type, checksum));
        chksum = blCrc32(accumulator, ((unsigned char *)&(params62->checksum)) + sizeof(params62->checksum), len);

        if (params62->checksum == chksum)
        {
            len = recSize - offsetof(devBoardParams62Type, waitTime);
            if (len < recSize) /* field position changed? */
            {
                memmove(&params->waitTime, &params62->waitTime, len);
                params->wlnParams.band = defaultParams.wlnParams.band;
                ret = BP_SUCCESS;
            }
        }
    }
    else /* initail version of 6.2? */
    {
        len = offsetof(devBoardParams62Type, wlnParams)  - offsetof(devBoardParams62Type, checksum) - sizeof(params62->checksum);
        accumulator = blCrc32(0, params62, offsetof(devBoardParams62Type, checksum));
        chksum = blCrc32(accumulator, ((unsigned char *)&(params62->checksum)) + sizeof(params62->checksum), len);

        if (params62->checksum == chksum)
        {
            len = offsetof(devBoardParams62Type, wlnParams) - offsetof(devBoardParams62Type, waitTime);
            if (len < sizeof *params62) /* field position changed? */
            {
                memmove(&params->waitTime, &params62->waitTime, len);  
                memcpy(&params->wlnParams, &defaultParams.wlnParams, sizeof params->wlnParams);
                ret = BP_SUCCESS;
            }
        }
    }

    return ret;    
}

/* function first tries to use the record size to calculate the checksum, if
   it fails then it tries to use old record size to calulate the checksum. 
   This is required because in older versions we didn't update the record
   size correctly */
int convertFromOlderVersionParams(devBoardParamsType * params)
{
    int retVal = BP_FAILURE;
    unsigned long recSize = params->sizeOfRecord;

    customizeUseDefaultParams(&defaultParams);

    /* try with original record size */
    if ((recSize <= sizeof *params) && (recSize >= offsetof(devBoardParamsType, snmpEngineBoots)))
    {
        if (params->checksum == isValidOlderVersionParams(params, recSize))
        {
            retVal = BP_SUCCESS;
            if (recSize < sizeof defaultParams)
            {
                memcpy ((BYTE *)params+recSize, (BYTE *)(&defaultParams)+recSize, sizeof defaultParams - recSize);
            }
            goto done_old_params;
        }
    }

    /* Check if it is from NET+OS 6.2 */
    if ((retVal = convertFrom62Params(params)) == BP_SUCCESS)
        goto done_old_params;

    /* Check if it is from NET+OS 6.1 */
    if ((retVal = convertFrom61Params(params)) == BP_SUCCESS)
        goto done_old_params;

    /* Check if it is from NET+OS 6.0 */
    if ((retVal = convertFrom60Params(params)) == BP_SUCCESS)
        goto done_old_params;

    /* finally check if it is 5.0 */    
    if ((retVal = convertFrom500Params(params)) == BP_SUCCESS)
        goto done_old_params;

    done_old_params:
    if (retVal == BP_SUCCESS)
    {
        int i;

        /* default the rest of the fields */
        for (i = CONFIG_ACE_MAX_INTERFACES_OLD; i < CONFIG_ACE_MAX_INTERFACES; i++)
        {
            memcpy(&params->aceConfig.ace_interface_info[i], &defaultParams.aceConfig.ace_interface_info[i], sizeof (aceConfigInterfaceInfo));  
        }
        params->sizeOfRecord = sizeof *params;
        params->checksum = computeChecksum(params);
    }
        
    return retVal;
}

#define NVRAM_75_TO_63

#ifdef NVRAM_75_TO_63
#define SIZE_OF_NETOS75_NVRAM 		4528
#define OFFSET_OF_NETOS75_CHECKSUM 	28
#define SIZE_OF_NETOS75_CHECKSUM 	(sizeof(unsigned long))
#define OFFSET_OF_NETOS75_MAC 		428

static unsigned long computeChecksumNetOS75(unsigned char * params)
{
    WORD32 accumulator = blCrc32(0, params, OFFSET_OF_NETOS75_CHECKSUM);
    
    return (blCrc32(accumulator, &params[OFFSET_OF_NETOS75_CHECKSUM + SIZE_OF_NETOS75_CHECKSUM],
        SIZE_OF_NETOS75_NVRAM - OFFSET_OF_NETOS75_CHECKSUM - SIZE_OF_NETOS75_CHECKSUM));
}

int convertFrom75Params(void)
{
    int ret = BP_FAILURE;
    int ccode;
    char paramsNetOS75[SIZE_OF_NETOS75_NVRAM+1];
    unsigned long * pChecksumNetOS75 = (unsigned long *)&paramsNetOS75[OFFSET_OF_NETOS75_CHECKSUM];

    /*
     * NetOS7.5 NVRAM is bigger than current (NetOS6.3) NVRAM, so we can't re-use 
     * previously read 'devBoardParamsType * params' and have to read again from NVRAM
     */
    ccode = NANVRead (START_OF_DEV_BOARD_PARAMETERS, paramsNetOS75, SIZE_OF_NETOS75_NVRAM);

    if (ccode == 0 && computeChecksumNetOS75(paramsNetOS75) == *pChecksumNetOS75)
    {
        // Recover MAC Address
        memcpy(&defaultParams.ethernetMac, &paramsNetOS75[OFFSET_OF_NETOS75_MAC], 6);

        ret = BP_SUCCESS;
    }

    return ret;
}

int convertFromNewerVersionParams(devBoardParamsType * params)
{
    int retVal = BP_FAILURE;

    customizeUseDefaultParams(&defaultParams);

    /* Check if it is from NET+OS 7.5 */
    retVal = convertFrom75Params();

    if (retVal == BP_SUCCESS)
    {
        memcpy (params, &defaultParams, sizeof (defaultParams));

        params->sizeOfRecord = sizeof *params;
        params->checksum = computeChecksum (params);
    }
        
    return retVal;
}
#endif

/*ad*
 * Reads the contents of the section of NVRAM that contains the development board's startup parameters.
 *
 * @external
 * @category Board_Params
 * @since 6.0
 *
 * @param params 	Pointer to structure with development board parameters
 *
 * @return 0		Successfully read.
 * @return -1		Unable to read parameters
 *
 * @see @link devBoardParamsType
*ad*/
int customizeReadDevBoardParams(devBoardParamsType * params)
{
    int result = BP_FAILURE;
    int ccode;

    NA_ASSERT(params != NULL);
    
    if (params == NULL)
    {
        goto done;
    }

    if (!NAGetAppUseNvram())  /* return the APP_USE_NVRAM value */
    {
        result = BP_SUCCESS;
        goto use_default;
    }

    ccode = NANVRead (START_OF_DEV_BOARD_PARAMETERS, (char *) params, sizeof (devBoardParamsType));

    if (ccode == 0)
    {    
        if (computeChecksum(params) != params->checksum)
        {
#ifdef NVRAM_75_TO_63
            if (convertFromNewerVersionParams(params) == BP_SUCCESS)
            {
                customizeWriteDevBoardParams (params);
                result = BP_SUCCESS;
                goto done;
            }
#endif
            if (convertFromOlderVersionParams(params) == BP_SUCCESS)
            {
                customizeWriteDevBoardParams (params);
                result = BP_SUCCESS;
                goto done;
            }
        }
        else
        {
            result = BP_SUCCESS;
            goto done;
        }
    }
    
    use_default:    
    customizeUseDefaultParams(&defaultParams);
    memcpy (params, &defaultParams, sizeof (defaultParams));
    params->sizeOfRecord = sizeof *params;
    params->checksum = computeChecksum (params);
    if (NAGetAppUseNvram())
    {
        customizeWriteDevBoardParams (params);
    }

    done:
    return result;
}




/*ad*
 * Writes development board parameters to NVRAM.
 *
 * @external
 * @category Board_Params
 * @since 6.0
 *
 * @param params 	Pointer to structure with development board parameters
 *
 * @return 0		The development board paremters were successfully written.
 * @return -1		Unable to write to NVRAM
 *
 * @see @link devBoardParamsType
*ad*/
int customizeWriteDevBoardParams(devBoardParamsType * params)
{
    NA_ASSERT(params != NULL);
    
    if (params != NULL)
    {
        params->checksum = computeChecksum(params);

	    if (NAGetAppUseNvram())  /* return the APP_USE_NVRAM value */
        {
            return NANVWrite (START_OF_DEV_BOARD_PARAMETERS, (char *) params, sizeof (devBoardParamsType));
        }
    }

    return -1;
}


/*ad*
 * Retrieves the IP address values (address, subnet maske and
 * default gateway) from NVRAM. The caller is then able to make a determination
 * if a valid IP address has been retrieved and stored in NVRAM.
 *
 *
 * Zero values indicate that the addresses have not been initialized.
 *
 * @external
 * @category Board_Params
 * @since 6.0
 *
 * @param ipptr		Pointer to destination of the retrieved IP address.
 * @param subptr	Pointer to destination of the retrieved subnet mask.
 * @param defgwptr	Pointer to the destination of the retrieved default gateway.
*ad*/
void customizeGetIPParameters(unsigned long * ipptr, unsigned long * subptr, unsigned long * defgwptr)
{
    configAceStaticInfo config;

    NA_ASSERT(ipptr != NULL);
    NA_ASSERT(subptr != NULL);
    NA_ASSERT(defgwptr != NULL);
    memset( (void*) &config, 0, sizeof(config));

    customizeAceGetStaticConfig(BP_ETH_INTERFACE, &config);    

    *ipptr = config.ip_address;
    *subptr = config.subnet_mask;
    *defgwptr = config.gateway;
}


/*ad*
 * Saves IP address parameters to NVRAM. 
 *
 * @external
 * @category Board_Params
 * @since 6.0
 *
 * @param ipaddr	IP address to save
 * @param submask	Subnet mask to save
 * @param defgw		Default gateway to save
*ad*/
void customizeSaveIPParameters(unsigned long ipaddr, unsigned long submask, unsigned long defgw)
{
    configAceStaticInfo config;

    NA_ASSERT(ipaddr > 0);
    NA_ASSERT(submask > 0);
    NA_ASSERT(defgw > 0);
    memset( (void*) &config, 0, sizeof(config));
    customizeAceGetStaticConfig(BP_ETH_INTERFACE, &config);    

    config.ip_address = ipaddr;
    config.subnet_mask = submask;
    config.gateway = defgw;
    config.isConfigValid = TRUE;
    config.isEnabled = TRUE;

    customizeAceSetStaticConfig(BP_ETH_INTERFACE, &config);            /* write updated params*/
}


void customizeSaveEnable(BOOLEAN mode)
{
   configAceStaticInfo static_config;
   configAceDhcpInfo   dhcp_config;

   customizeAceGetStaticConfig(BP_ETH_INTERFACE, &static_config);
   customizeAceGetDhcpConfig(BP_ETH_INTERFACE, &dhcp_config);

   static_config.isEnabled = !mode;
   dhcp_config.isEnabled = mode;

   customizeAceSetStaticConfig(BP_ETH_INTERFACE, &static_config);
   customizeAceSetDhcpConfig(BP_ETH_INTERFACE, &dhcp_config);
}





